package net.theender.ldbridge;

import net.luckperms.api.LuckPerms;
import net.luckperms.api.model.user.User;
import net.luckperms.api.node.types.MetaNode;
import org.apache.commons.lang.RandomStringUtils;
import org.asynchttpclient.Response;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.command.*;
import org.bukkit.entity.Player;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

public class CommandLink implements CommandExecutor, TabCompleter {
    public TabCompleter onTabComplete;
    LuckPerms luckPerms;
    LiizziisDiscordBridge plugin;

    public CommandLink(LiizziisDiscordBridge main) {
        luckPerms = main.luckperms;
        plugin = main;

    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if (sender instanceof Player) {
            Player player = (Player) sender;
            User user = luckPerms.getPlayerAdapter(Player.class).getUser(player);
            String discordID = user.getCachedData().getMetaData().getMetaValue("discord_id");
            if (discordID == null) {
                if (args.length == 0) {return false; }
                switch (args[0]) {
                    case "start":
                        this.plugin.getLogger().info("THEY WANT TO START AAAAAAAAAAA");
                        player.sendMessage(ChatColor.LIGHT_PURPLE+"Starting link process, one moment...");
                        Bukkit.getScheduler().runTaskAsynchronously(this.plugin, () -> {startLinkProcess(player);});
                        return true;

                    case "confirm":
                        if (args.length == 1) {
                            player.sendMessage(ChatColor.RED+"Missing confirmation code");
                            return false;
                        } else {
                            Bukkit.getScheduler().runTaskAsynchronously(this.plugin, () -> {confirmLinkProcess(player, user, args[1]);});
                            return true;

                        }

                    default:
                        player.sendMessage(ChatColor.RED+"Unrecognised subcommand(s)");
                        return false;
                }
            }
            if (discordID != null) {
                player.sendMessage(
                        ChatColor.RED + "You already have a Discord ID linked: " + ChatColor.GREEN + ChatColor.BOLD + discordID + ChatColor.RESET,
                        ChatColor.RED + "If you wish to link a different Discord ID, please contact the moderation team!");

                return true;
            }
        } else if (sender instanceof ConsoleCommandSender) {
            ConsoleCommandSender console = (ConsoleCommandSender) sender;
            console.sendMessage("boobies");
            return true;
        }
        return false;
    }


    public void startLinkProcess(Player player) {
        String linkCode = RandomStringUtils.random(6, true, false);
        plugin.getLogger().info(linkCode);
        // do call to Evey's discord bit here
        try {
            JSONObject data = new JSONObject();
            data.put("linkCode", linkCode);
            data.put("playerID", player.getUniqueId());
            String url = String.format("http://%s/mclink", plugin.getConfig().getString("link_host", "127.0.0.1"));
            Response response = utils.makePostRequest(url, data);
            // Return 200 for OK (Evey's stored the code), anything else to indicate error
            if (response.getStatusCode() == 204) {
                player.sendMessage(ChatColor.GREEN + "Account linking started, your link code is: " + ChatColor.BLUE + ChatColor.UNDERLINE + linkCode,
                        ChatColor.GREEN + "Now go over to #floofycraft on the discord and type: ",
                        ChatColor.GOLD + "?link_mc " + linkCode,
                        ChatColor.DARK_GREEN + "Evey will give you a code and instructions to paste back here :)");
            } else if (response.getStatusCode() == 200) {
                // Evey already has a session open for this player and has given back the code, lets just give that to the player
                player.sendMessage(ChatColor.GREEN+"Account linking started, your link code is: "+ChatColor.BLUE+ChatColor.UNDERLINE+response.getResponseBody(),
                        ChatColor.GREEN+"Now go over to #floofycraft on the discord and type: ",
                        ChatColor.GOLD+"?link_mc "+response.getResponseBody(),
                        ChatColor.DARK_GREEN+"Evey will give you a code and instructions to paste back here :)");
            } else {
                player.sendMessage(ChatColor.RED+"Uhoh, something went wrong. Poke Mama Liizzii!");
                plugin.getLogger().warning(String.format("Got %d when trying to set up code for %s", response.getStatusCode(), player.getDisplayName()));
                plugin.getLogger().warning(response.getResponseBody());
            }

        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
        }
    }

    public void confirmLinkProcess(Player player, User user, String code) {
        // Do confirmation stuff here....
        try {
            JSONObject data = new JSONObject().put("confirmCode", code).put("playerID", player.getUniqueId());
            String url = String.format("http://%s/mclink/confirm", plugin.getConfig().getString("link_host", "127.0.0.1"));
            Response response = utils.makePostRequest(url, data);
            switch (response.getStatusCode()) {
                case 200:
                    String discordID = response.getResponseBody();
//                    player.sendMessage(discordID);
                    MetaNode node = MetaNode.builder("discord_id", discordID).build();
                    user.data().add(node);
                    luckPerms.getUserManager().saveUser(user);
                    player.sendMessage(ChatColor.GREEN+"Linking successful!");
                    Thread.sleep(5000);
                    player.sendMessage(ChatColor.DARK_GREEN+"You can use the server switch command to jump back to main now :)");
                    // maybe do the above a different way?
                    break;

                case 404:
                    player.sendMessage(ChatColor.RED+"That code doesn't look valid, did you copy it exactly?");
                    break;

                default:
                    player.sendMessage(ChatColor.GOLD+"An error occured! Poke Mama Liizzii");
                    plugin.getLogger().warning(String.format("for %s: %s|%s", player.getDisplayName(), response.getStatusCode(), response.getResponseBody()));
            }
        } catch (InterruptedException|ExecutionException e) {
            e.printStackTrace();
        }

    }


    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        if (command.getName().equalsIgnoreCase("ldb")) {
            List<String> l = new ArrayList<String>();
            if (args.length==1) {
                // Most likely they've just typed "/ldb" only at this point
                l.add("start");
                l.add("confirm");
            } else if (args.length==2 && !args[0].equalsIgnoreCase("start")) {
                l.add("<CODE>");
            }
            return l;
        }

        return null;
    }
}
