package net.theender.ldbridge;

import net.luckperms.api.cacheddata.CachedMetaData;
import net.luckperms.api.messaging.MessagingService;
import net.luckperms.api.model.group.Group;
import net.luckperms.api.model.user.User;
import net.luckperms.api.model.user.UserManager;
import net.luckperms.api.node.NodeType;
import net.luckperms.api.node.types.InheritanceNode;
import org.asynchttpclient.Response;
import org.bukkit.*;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.*;
import org.json.JSONArray;
import org.json.JSONObject;

import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ThreadLocalRandom;
import java.util.logging.Logger;
import java.util.stream.Collectors;

public class LDListener implements Listener {
    net.theender.ldbridge.LiizziisDiscordBridge MainClass;
    Logger logger;
    FileConfiguration config;
    Location WorldSpawn;

    Map<Player, Location> playerRespawnLocations;

    public LDListener(LiizziisDiscordBridge main) {
        MainClass = main;
        logger = this.MainClass.getLogger();
        this.config = MainClass.config;
        this.playerRespawnLocations = new HashMap<Player, Location>();
        try {
            WorldSpawn = Objects.requireNonNull(MainClass.getServer().getWorld("world")).getSpawnLocation();
        } catch (NullPointerException e) {
            e.printStackTrace();
        }
    }

    @EventHandler
    public void onAsyncPreLogin(AsyncPlayerPreLoginEvent event) {
        if (!MainClass.DiscordLinking) { return; }
        this.logger.info("Nyaaa!?!?!?!");
        this.logger.info(String.valueOf(event.isAsynchronous()));
        this.logger.info(String.valueOf(event.getUniqueId()));
        UserManager userManager = this.MainClass.luckperms.getUserManager();
        User user = userManager.loadUser(event.getUniqueId()).join();
        if (user == null && this.config.getBoolean("kick-no-discord")) {
            // With loadUser, this will most likely never fire unless LP has an issue
            event.disallow(AsyncPlayerPreLoginEvent.Result.KICK_OTHER, "User data doesn't exist.");
            return;
        }
        String discordID = user.getCachedData().getMetaData().getMetaValue("discord_id");
        if (discordID == null) {
            if (this.MainClass.getConfig().getBoolean("kick-no-discord")) {
                // If the user doesn't have a discord ID in their metadata, then optionally boot them off.
                // Allows them to be booted over to a "holding" server for them to link their discord info
                event.disallow(AsyncPlayerPreLoginEvent.Result.KICK_WHITELIST, "No existing Discord ID link found");
            }
        } else {
            try { // might wanna run this in another task // nah, this is pretty quick. only slow thing might be ban information but I can have Evey cache that herself
                Response response = utils.makeGetRequest(String.format("http://%s/userinfo/%s/%s", config.getString("link_host", "127.0.0.1"), config.getString("discord_guild"),discordID));
                switch (response.getStatusCode()) {
                    case 200:
                        // 200 means they're on the server.
                        JSONObject data = new JSONObject(response.getResponseBody());
                        Bukkit.getScheduler().runTaskAsynchronously(this.MainClass, new Runnable() {
                            @Override
                            public void run() {
                                Set<InheritanceNode> userGroups = user.getNodes(NodeType.INHERITANCE).stream().map(NodeType.INHERITANCE::cast).collect(Collectors.toSet());

                                // This entire section feels really fucking stupid, but I can't work out how to do it elegantly
                                ArrayList<String> rolesOnDiscord = new ArrayList<String>();
                                JSONArray roles = data.getJSONArray("roles");
                                if (roles != null) {
                                    int len = roles.length();
                                    for (int i=0;i<len;i++){
                                        rolesOnDiscord.add( roles.get(i).toString() );
                                    }
                                }
                                // End of stupid section

                                Set<Group> groupList = MainClass.luckperms.getGroupManager().getLoadedGroups();
                                boolean userChanged = false;
                                for (Group group : groupList) {
                                    String groupRoleID = config.getString(String.format("discord_roles.%s", group.getName()), null);
                                    if (groupRoleID == null) {continue;}
                                    InheritanceNode groupNode = InheritanceNode.builder(group).build();
                                    if (rolesOnDiscord.contains(groupRoleID) && !userGroups.contains(groupNode)) {
                                        user.data().add(groupNode);
                                        userChanged = true;
                                    } else if (!rolesOnDiscord.contains(groupRoleID) && userGroups.contains(groupNode)) {
                                        user.data().remove(groupNode);
                                        userChanged = true;
                                    }
                                }
                                if (userChanged) {
                                    CompletableFuture<Void> task = userManager.saveUser(user);
                                    task.thenRunAsync(() -> {
                                        Optional<MessagingService> messagingService = MainClass.luckperms.getMessagingService();
                                        messagingService.ifPresent(service -> service.pushUserUpdate(user));
                                    });
                                }
                            }
                        });
                        break;
                    case 403:
                        // User is banned
                        event.disallow(AsyncPlayerPreLoginEvent.Result.KICK_BANNED, "You've been banned from the discord!");
                        Bukkit.getBanList(BanList.Type.NAME).addBan(event.getName(), "Banned on discord.", null, "Discord");
                        break;
                    case 404:
                        // user isn't on the discord, but not banned
                        event.disallow(
                                AsyncPlayerPreLoginEvent.Result.KICK_WHITELIST,
                                "You've previously linked a discord ID, but you're not on the discord server... " +
                                "\nJoin back to play again https://discord.onlyfurs.gay");
                        break;
                }
            } catch (ExecutionException | InterruptedException e) {
                e.printStackTrace();
            }

        }
    }

    @EventHandler
    public void onPlayerJoin(PlayerJoinEvent event) {
        // Check here if user has linked discord stuff

        Player player = event.getPlayer();
        JSONObject Data = new JSONObject();

        CachedMetaData metaData = MainClass.luckperms.getPlayerAdapter(Player.class).getMetaData(player);

        if (metaData.getMetaValue("discord_id") == null) {
            if (MainClass.DiscordLinking) {
                player.sendMessage(ChatColor.LIGHT_PURPLE + "You don't have a discord account linked. See the hologram for instructions.");
            }
            MainClass.getLogger().info(String.format("Player %s doesn't have a Discord ID linked.", player.getDisplayName()));
        } else {
            MainClass.getLogger().info(String.format("Player %s has a discord ID of %s", player.getDisplayName(), metaData.getMetaValue("discord_id")));
        }
        String discordID = metaData.getMetaValue("discord_id");
        String discordMention = (discordID != null) ? String.format("<@%s>", discordID) : "None";
        Data.put("content", String.format("[ __%s__ ] JOIN: %s (MC Username: `%s` | Discord ID: %s) just joined the server!",
                MainClass.Instance, player.getDisplayName(), player.getName(), discordMention));
        Data.put("allowed_mentions", MainClass.AllowedMentions);
        Bukkit.getScheduler().runTaskAsynchronously(MainClass, () -> {
            try {
                utils.makePostRequest(MainClass.getConfig().getString("join-part-hook"), Data);
            } catch (InterruptedException | ExecutionException e) {
                e.printStackTrace();
            }
        });
    }

    @EventHandler
    public void onPlayerQuit(PlayerQuitEvent event) {
        Player player = (Player) event.getPlayer();
        String discordID = this.MainClass.luckperms.getPlayerAdapter(Player.class).getUser(player).getCachedData().getMetaData().getMetaValue("discord_id");
        String discordMention = (discordID != null) ? String.format("<@%s>", discordID) : "None";
        JSONObject Data = new JSONObject();
        Data.put("content", String.format("[ __%s__ ] PART: %s (MC Username: `%s` | Discord ID: %s) just left the " +
                "server!\n`%s`",
                MainClass.Instance, event.getPlayer().getDisplayName(), event.getPlayer().getName(), discordMention, event.getQuitMessage()));
        Data.put("allowed_mentions", MainClass.AllowedMentions);
        Bukkit.getScheduler().runTaskAsynchronously(MainClass, () -> {
            try {
                utils.makePostRequest(MainClass.getConfig().getString("join-part-hook"), Data);
            } catch (InterruptedException | ExecutionException e) {
                e.printStackTrace();
            }
        });
    }

    @EventHandler
    public void onPlayerDeath(PlayerDeathEvent event) {
        if (this.WorldSpawn == null) {
            WorldSpawn = MainClass.getServer().getWorld("world").getSpawnLocation();
        }
        if (MainClass.MusicalSpawns) {
            int OldX = (int) WorldSpawn.getX();
            int OldZ = (int) WorldSpawn.getY();
            int NewX = ThreadLocalRandom.current().nextInt(OldX-1000, OldX+1000);
            int NewZ = ThreadLocalRandom.current().nextInt(OldZ-1000, OldZ+1000);
            int y = Objects.requireNonNull(WorldSpawn.getWorld()).getHighestBlockYAt(NewX, NewZ);

            playerRespawnLocations.put(event.getEntity(), new Location(WorldSpawn.getWorld(), NewX, y, NewZ));
        }
    }

    @EventHandler
    public void onAsyncPlayerChat(AsyncPlayerChatEvent event) {
        Player player = event.getPlayer();
        String MessageContent = String.format("[ __%s__ ] CHAT: %s (`%s`)>\n```%s```",
                MainClass.Instance, player.getDisplayName(), player.getName(), event.getMessage().replace("\\", "\\\\" ).replace("`", "\\`"));
        JSONObject Data = new JSONObject();
        Data.put("content", MessageContent);
        Data.put("allowed_mentions", MainClass.AllowedMentions);
        Bukkit.getScheduler().runTaskAsynchronously(MainClass, () -> {
            try {
                utils.makePostRequest(MainClass.getConfig().getString("chat-webhook"), Data);
            } catch (InterruptedException | ExecutionException e) {
                e.printStackTrace();
            }
        });
    }

    @EventHandler
    public void onPlayerRespawn(PlayerRespawnEvent event) {
        if (this.WorldSpawn == null) {
            WorldSpawn = MainClass.getServer().getWorld("world").getSpawnLocation();
        }
        if (MainClass.MusicalSpawns) {
            this.logger.info(String.format("Is bed? %s | Location: %s", event.isBedSpawn(), event.getRespawnLocation()));
            Location NewRespawnLocation;
            try {
                NewRespawnLocation = playerRespawnLocations.get(event.getPlayer());
            } catch (NullPointerException e) {
                NewRespawnLocation = WorldSpawn;
            }

            this.logger.info(String.format("%s", event.getRespawnLocation()));
            this.logger.info(String.format("%s", NewRespawnLocation));
            if (!event.isBedSpawn()) {
                event.setRespawnLocation(NewRespawnLocation);
            }
        }
    }

}

